@@ -17,8 +17,16 @@
  */
 
 #define _GNU_SOURCE
+#ifdef _MSC_VER
+#include <BaseTsd.h>
+typedef SSIZE_T ssize_t;
+#define alloca _alloca
+#else
 #include <unistd.h>
+#endif
+#include <string.h>
 #include <fcntl.h>
+#include <string.h>
 #include "sha1.h"
 
 static inline int sha1_file(char *filename, sha1_digest *digest)
@@ -28,7 +36,7 @@ static inline int sha1_file(char *filename, sha1_digest *digest)
 	int fd; ssize_t n;
 	struct sha1_ctx ctx;
 
-#ifdef WIN32
+#ifndef O_CLOEXEC
 	fd = open(filename, O_RDONLY);
 #else
 	fd = open(filename, O_RDONLY | O_CLOEXEC);
@@ -80,12 +88,15 @@ CAMLprim value stub_sha1_update(value ctx, value data, value ofs, value len)
 CAMLprim value stub_sha1_update_bigarray(value ctx, value buf)
 {
 	CAMLparam2(ctx, buf);
+	struct sha1_ctx ctx_dup;
 	unsigned char *data = Data_bigarray_val(buf);
 	size_t len = Bigarray_val(buf)->dim[0];
 
+	ctx_dup = *GET_CTX_STRUCT(ctx);
 	caml_release_runtime_system();
-	sha1_update(GET_CTX_STRUCT(ctx), data, len);
+	sha1_update(&ctx_dup, data, len);
 	caml_acquire_runtime_system();
+	*GET_CTX_STRUCT(ctx) = ctx_dup;
 
 	CAMLreturn(Val_unit);
 }
